package com.persagy.dmp.rwd.sharding;

import com.persagy.dmp.rwd.config.JpaSqlExecuteContext;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.resource.jdbc.spi.StatementInspector;

/**
 * @author: yaoll
 * @date: 2020-12-20
 * @verison: 1.0
 */
@Slf4j
public class JpaSqlStatementInspector implements StatementInspector {

	private final static String TABLE_DEF_CLASS = "rwd_def_class";
	private final static String TABLE_DEF_FUNCID = "rwd_def_funcid";

	private final static String TABLE_SCHEMA = "rwd_schema";
	private final static String TABLE_SCHEMA_CLASS = "rwd_schema_class";
	private final static String TABLE_SCHEMA_FUNCID = "rwd_schema_funcid";
	private final static String TABLE_SCHEMA_PROJECT = "rwd_schema_project";

	private final static String TABLE_APP_CLASS = "rwd_app_class";
	private final static String TABLE_APP_FUNCID = "rwd_app_funcid";

	private final static String TABLE_INSTANCE_GRAPH = "rwd_graph";
	private final static String TABLE_INSTANCE_OBJEC = "rwd_object";
	private final static String TABLE_INSTANCE_INSTANCE = "rwd_relation";


	@Override
	public String inspect(String sql) {
		// 只拦截dml
		if (sql.startsWith("select") || sql.startsWith("update") || sql.startsWith("delete") || sql.startsWith("insert")) {
			log.info("sql inspector: {}", sql);

			sql = execute(sql, TABLE_DEF_CLASS, groupTableStrategy);
			sql = execute(sql, TABLE_DEF_FUNCID, groupTableStrategy);

			sql = execute(sql, TABLE_SCHEMA, groupTableStrategy);
			sql = execute(sql, TABLE_SCHEMA_CLASS, groupTableStrategy);
			sql = execute(sql, TABLE_SCHEMA_FUNCID, groupTableStrategy);
			sql = execute(sql, TABLE_SCHEMA_PROJECT, groupTableStrategy);

			sql = execute(sql, TABLE_APP_CLASS, groupTableStrategy);
			sql = execute(sql, TABLE_APP_FUNCID, groupTableStrategy);

			sql = execute(sql, TABLE_INSTANCE_GRAPH, groupProjectTableStrategy);
			sql = execute(sql, TABLE_INSTANCE_OBJEC, groupProjectTableStrategy);
			sql = execute(sql, TABLE_INSTANCE_INSTANCE, groupProjectTableStrategy);
		}
		log.info("sql result: {}", sql);
		return sql;
	}

	private String execute(String sql, String logicName, TableStrategy strategy) {
		if (sql.indexOf(" " + logicName + " ") > -1) {
			String actualName = strategy.strategy(logicName);
			sql = sql.replaceAll(" " + logicName + " ", " " + actualName + " ");
			log.info("sql actual table: {}", actualName);
		}
		return sql;
	}

	public interface TableStrategy {
		String strategy(String logicName);
	}

	private TableStrategy groupTableStrategy = logicName -> {
		String groupCode = JpaSqlExecuteContext.getGroupCode();
		if (groupCode == null || "0".equals(groupCode)) {
			return logicName;
		}
		return logicName + "_" + groupCode.toLowerCase().trim();
	};

	private TableStrategy groupProjectTableStrategy = logicName -> {
		String groupCode = JpaSqlExecuteContext.getGroupCode();
		String projectId = JpaSqlExecuteContext.getProjectId();
		if (groupCode == null || projectId == null || "0".equals(groupCode) || "0".equals(projectId)) {
			return logicName;
		}
		return logicName + "_" + groupCode.toLowerCase().trim() + "_" + projectId.toLowerCase().trim();
	};

}
